# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-03 19:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0002_add_defaults_to_blank_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='matchup',
            name='featured',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-03 19:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='matchup',
            name='description',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='matchup',
            name='email',
            field=models.EmailField(blank=True, db_index=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='matchup',
            name='period',
            field=models.CharField(blank=True, default='', max_length=32),
        ),
    ]



from django.conf import settings
from django.core.checks import Warning, register


@register()
def check_installed_apps(app_configs, **kwargs):
    if 'django.contrib.postgres' not in settings.INSTALLED_APPS:
        return [
            Warning(
                'django.contrib.postgres is not included in INSTALLED_APPS. '
                'This is required for Postgres full text search.',
                hint='add django.contrib.postgres to INSTALLED_APPS',
                id='bomojo.W001',
            )
        ]

    return []



from django.test import RequestFactory, override_settings

from bomojo.matchups.forms import MatchupForm
from bomojo.tests import TestCase


class MatchupFormTest(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.create_movie('Foo')
        cls.create_movie('Foo 2', external_id='foo2')
        cls.create_movie('Foo 3', external_id='foo3')
        cls.create_movie('Bar')

    def setUp(self):
        super().setUp()
        self.user = self.create_user('joe')
        self.request = RequestFactory().post('/matchup')

        # Simulate the user being authenticated.
        self.request.user = self.user

    def test_happy_path(self):
        form = MatchupForm(self.request, {
            'title': 'foo trilogy',
            'description': 'The entire foo trilogy',
            'movies': ['foo', 'bar'],
            'period': '30d'
        })

        self.assertTrue(form.is_valid(), form.errors)

        matchup = form.save()

        self.assertEqual(matchup.title, 'foo trilogy')
        self.assertEqual(matchup.description, 'The entire foo trilogy')
        self.assertEqual(matchup.movies, ['foo.htm', 'bar.htm'])
        self.assertEqual(matchup.period, '30d')

    def test_minimal_fields(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': ['foo', 'bar']
        })

        self.assertTrue(form.is_valid(), form.errors)

        matchup = form.save()

        self.assertEqual(matchup.title, 'foo vs bar')
        self.assertEqual(matchup.description, '')
        self.assertEqual(matchup.movies, ['foo.htm', 'bar.htm'])
        self.assertEqual(matchup.period, '')

    def test_parses_movie_ids(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': ['foo.htm', 'bar.htm']
        })

        self.assertTrue(form.is_valid(), form.errors)

        matchup = form.save()

        self.assertEqual(matchup.title, 'foo vs bar')
        self.assertEqual(matchup.description, '')
        self.assertEqual(matchup.movies, ['foo.htm', 'bar.htm'])
        self.assertEqual(matchup.period, '')

    def test_title_cannot_be_omitted(self):
        form = MatchupForm(self.request, {
            'description': 'The entire foo trilogy',
            'movies': ['foo1', 'foo2', 'foo3']
        })

        self.assertFalse(form.is_valid())
        self.assertIn('title', form.errors)
        self.assertIn('required', *form.errors['title'])

    def test_title_cannot_be_blank(self):
        form = MatchupForm(self.request, {
            'title': '',
            'description': 'The entire foo trilogy',
            'movies': ['foo1', 'foo2', 'foo3']
        })
        self.assertFalse(form.is_valid())
        self.assertIn('title', form.errors)
        self.assertIn('required', *form.errors['title'])

    def test_movies_cannot_be_omitted(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar'
        })
        self.assertFalse(form.is_valid())
        self.assertIn('movies', form.errors)
        self.assertIn('required', *form.errors['movies'])

    def test_movies_cannot_be_empty(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': []
        })
        self.assertFalse(form.is_valid())
        self.assertIn('movies', form.errors)
        self.assertIn('required', *form.errors['movies'])

    def test_movies_must_be_valid(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs quux',
            'movies': ['foo', 'quux']
        })
        self.assertFalse(form.is_valid())
        self.assertIn('movies', form.errors)

    def test_period_must_be_valid_if_provided(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': ['foo', 'bar'],
            'period': 'blah'
        })
        self.assertFalse(form.is_valid())
        self.assertIn('period', form.errors)
        self.assertIn('valid', *form.errors['period'])

    def test_featured_only_allowed_for_featured_contributors(self):
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': ['foo', 'bar'],
            'featured': True
        })
        self.assertFalse(form.is_valid())
        self.assertIn('featured', form.errors)

        self.user.is_staff = True
        self.user.save()

        # Re-initialize form to run validation again with updated data.
        form = MatchupForm(self.request, {
            'title': 'foo vs bar',
            'movies': ['foo', 'bar'],
            'featured': True
        })
        self.assertTrue(form.is_valid())



from bomojo.tests import TestCase



class MovieTestCase(TestCase):
    def setUp(self):
        self.movie = self.create_movie(title='Unbreakable',
                                       external_id='unbreakable')

    def test_str(self):
        self.assertEqual('Unbreakable (unbreakable)', str(self.movie))

    def test_repr(self):
        self.assertEqual('<Movie: Unbreakable (unbreakable)>', repr(self.movie))



"""
bomojo URL Configuration
"""

from django.conf.urls import include, url

urlpatterns = [
    url(r'^movies/', include('bomojo.movies.urls')),
    url(r'^matchups/', include('bomojo.matchups.urls')),
]



from django.core.management.base import BaseCommand
from bomojo.movies.models import PriceIndex

import pycpi

class Command(BaseCommand):
    help = 'Loads CPI data from the BLS API'

    def handle(self, *args, **options):
        months_by_name = {
            'January': 1,
            'February': 2,
            'March': 3,
            'April': 4,
            'May': 5,
            'June': 6,
            'July': 7,
            'August': 8,
            'September': 9,
            'October': 10,
            'November': 11,
            'December': 12
        }

        latest_data = PriceIndex.objects.order_by('year', 'month').last()
        start_year = latest_data.year if latest_data is not None else 1920

        print('Loading CPI data from %d' % start_year)
        for year, month, value in pycpi.get_data(start_year):
            price_index, created = PriceIndex.objects.get_or_create(
                year=year, month=months_by_name[month], value=value)
            if created:
                print(price_index)



from bomojo.backends import get_movie_backend


def render_movie(movie, backend=None):
    backend = backend or get_movie_backend()

    return {
        'title': movie.title,
        'movie_id': backend.format_external_id(movie.external_id)
    }


def render_box_office(box_office, movie_id):
    return {
        'title': box_office.get('title'),
        'movie_id': movie_id,
        'href': box_office.get('href'),
        'box_office': [{
            'gross': int(result['gross']),
            'cumulative': int(result['cumulative']),
            'rank': result['rank'],
            'theaters': result['theaters'],
            'date': result['date'],
            'day': result['day'],
        } for result in box_office.get('box_office', [])]
    }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-03-11 02:39
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0003_matchup_featured'),
    ]

    operations = [
        migrations.AlterField(
            model_name='matchup',
            name='period',
            field=models.CharField(blank=True, default='', max_length=32, validators=[django.core.validators.RegexValidator('^\\d+[dw]$')]),
        ),
    ]



import os

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase as DjangoTestCase

from bomojo.backends import get_movie_backend

TEST_DATA_DIR = os.path.join(settings.BASE_DIR, 'tests', 'data')


class TestCase(DjangoTestCase):
    @staticmethod
    def create_user(username, **kwargs):
        User = get_user_model()

        user_data = {
            'username': username,
            'first_name': username.capitalize(),
            'email': f'{username}@example.com'
        }

        password = kwargs.pop('password', 'top secret')

        user_data.update(kwargs)

        user = User(**user_data)
        user.set_password(password)
        user.save()

        return user

    @staticmethod
    def create_movie(title, **kwargs):
        from bomojo.movies.models import Movie

        backend = kwargs.pop('backend', None) or get_movie_backend()

        movie_data = {
            'title': title,
            'external_id': backend.parse_movie_id(title.lower())
        }

        movie_data.update(kwargs)

        return Movie.objects.create(**movie_data)

    @staticmethod
    def create_matchup(user, movies, **kwargs):
        from bomojo.matchups.models import Matchup

        backend = kwargs.pop('backend', None) or get_movie_backend()

        matchup_data = {
            'user': user,
            'title': ' vs '.join(movies),
            'description': 'matchup between %s' % ' and '.join(movies),
            'movies': [backend.parse_movie_id(movie_id)
                       for movie_id in movies],
            'period': '30d'
        }

        matchup_data.update(kwargs)

        return Matchup.objects.create(**matchup_data)

    def create_image(self, name, file='baby.png'):
        with open(os.path.join(TEST_DATA_DIR, file), 'rb') as f:
            content = f.read()

        image = SimpleUploadedFile(name=name,
                                   content=content,
                                   content_type='image/png')

        self.addCleanup(lambda: os.remove(
            os.path.join(TEST_DATA_DIR, 'matchups', image.name)))

        return image



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-05 01:11
from __future__ import unicode_literals

from django.db import migrations, models
import partial_index


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0006_matchup_updated_on'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='matchup',
            index=models.Index(fields=['user', 'created_on'], name='matchups_ma_user_id_b8c76c_idx'),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=models.Index(fields=['user', 'updated_on'], name='matchups_ma_user_id_754211_idx'),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=partial_index.PartialIndex(fields=['created_on'], name='matchups_ma_created_250a2c_partial', unique=False, where='featured = true', where_postgresql='', where_sqlite=''),
        ),
        migrations.AddIndex(
            model_name='matchup',
            index=partial_index.PartialIndex(fields=['updated_on'], name='matchups_ma_updated_f84f32_partial', unique=False, where='featured = true', where_postgresql='', where_sqlite=''),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-29 04:08
from __future__ import unicode_literals

from django.db import migrations, models


def populate_updated_on(apps, schema_editor):
    Matchup = apps.get_model('matchups', 'Matchup')

    for matchup in Matchup.objects.filter(updated_on__isnull=True):
        matchup.updated_on = matchup.created_on
        matchup.save()


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0005_matchup_remove_email_add_user'),
    ]

    operations = [
        # Introduce the updated_on field as nullable at first.
        migrations.AddField(
            model_name='matchup',
            name='updated_on',
            field=models.DateTimeField(null=True),
        ),

        # Set updated_on = created_on for existing matchups.
        migrations.RunPython(populate_updated_on),

        # Now add a non-null constraint.
        migrations.AlterField(
            model_name='matchup',
            name='updated_on',
            field=models.DateTimeField(auto_now=True),
        ),
    ]



import json


class JSONMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if 'application/json' in (request.META.get('CONTENT_TYPE') or ''):
            request.JSON = json.loads(request.body.decode('utf-8'))

        return self.get_response(request)



"""
Default Django settings
"""

# Movie search
MOVIE_MIN_SEARCH_LENGTH = 3
MOVIE_MAX_SEARCH_RESULTS = 20

# Matchup listing
DEFAULT_MATCHUP_SORTING = '-created_on'

# Avatars
DEFAULT_AVATAR_SIZE = 32
DEFAULT_AVATAR_STYLE = 'retro'

# Backend containing business logic for exposing functionality to users
USER_BACKEND = 'bomojo.backends.DefaultUserBackend'

# Backend containing business logic related to rendering of movies
MOVIE_BACKEND = 'bomojo.backends.DefaultMovieBackend'



from collections import OrderedDict
from decimal import Decimal
import hashlib
import re

from django.conf import settings

from bomojo import defaults

_money_quantization = Decimal('1.00')
_whitespace_pattern = re.compile(r'\s+')


def get_avatar_url(email, size=None, default=None):
    email = email.lower()
    size = size or get_setting('DEFAULT_AVATAR_SIZE')
    default = default or get_setting('DEFAULT_AVATAR_STYLE')

    digest = hashlib.new('md5', email.encode('utf-8')).hexdigest()
    return 'https://gravatar.com/avatar/%(digest)s?size=%(size)d&d=%(default)s' % {
        'digest': digest,
        'size': size,
        'default': default
    }


def get_setting(setting_name):
    return getattr(settings, setting_name, getattr(defaults, setting_name))


def deduplicate(values):
    """Returns a list of the same values but with duplicates removed

    Unlike list(set(values)), this function preserves the ordering of the
    original list.
    """
    return list(OrderedDict.fromkeys(values))


def split_on_whitespace(text):
    return _whitespace_pattern.split(text)


def format_money(value):
    if not isinstance(value, Decimal):
        value = Decimal(value)
    return value.quantize(_money_quantization)



from django.utils.module_loading import import_string

from bomojo.utils import get_setting


class AbstractUserBackend(object):
    """Abstract class defining required methods for any USER_BACKEND class"""
    def is_featured_contributor(self, user):
        """Indicate whether a user is allowed to make "featured" matchups"""
        raise NotImplementedError


class DefaultUserBackend(AbstractUserBackend):
    def is_featured_contributor(self, user):
        return user.is_staff


def get_user_backend():
    backend_name = get_setting('USER_BACKEND')
    backend_class = import_string(backend_name)
    return backend_class()


class AbstractMovieBackend(object):
    """Abstract class defining required methods for any MOVIE_BACKEND class

    Any movie backend needs to be able to perform lossless two-way conversions
    between external IDs provided by a service and values rendered to the end
    user. The idea is that a service may provide "ugly" IDs that shouldn't be
    rendered as-is, in which case the backend still needs to be able to produce
    the "ugly" ID from a formatted ID for requests back to the service.
    """
    def format_external_id(self, external_id):
        """Translate an external ID to something suitable for display"""
        raise NotImplementedError

    def parse_movie_id(self, movie_id):
        """Resolve an ID to a value recognized by an external service"""
        raise NotImplementedError


class DefaultMovieBackend(AbstractMovieBackend):
    def format_external_id(self, external_id):
        if external_id.endswith('.htm'):
            external_id = external_id[:-4]
        return external_id

    def parse_movie_id(self, movie_id):
        if not movie_id.endswith('.htm'):
            movie_id = f'{movie_id}.htm'
        return movie_id


def get_movie_backend():
    backend_name = get_setting('MOVIE_BACKEND')
    backend_class = import_string(backend_name)
    return backend_class()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.forms import ModelForm

from bomojo.backends import get_movie_backend, get_user_backend
from bomojo.matchups.models import Matchup
from bomojo.movies.models import Movie
from bomojo.utils import deduplicate


class MatchupForm(ModelForm):
    class Meta:
        model = Matchup
        fields = ['title', 'description', 'movies', 'period', 'featured']

    def __init__(self, request, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instance.user = request.user

    def clean_featured(self):
        if self.cleaned_data['featured']:
            user_backend = get_user_backend()
            if not user_backend.is_featured_contributor(self.instance.user):
                raise ValidationError('You must be a featured contributor to '
                                      'create featured matchups.',
                                      code='not_featured_contributor')
        return self.cleaned_data['featured']

    def clean_movies(self):
        movie_ids = deduplicate(self.cleaned_data['movies'])
        backend = get_movie_backend()
        movie_ids = [backend.parse_movie_id(movie_id)
                     for movie_id in movie_ids]
        movies = Movie.objects.filter(external_id__in=movie_ids)
        if len(movies) != len(movie_ids):
            raise ValidationError("One or more of those movies isn't "
                                  "recognized.")
        return movie_ids



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-02 11:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('movies', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='movie',
            name='title',
            field=models.CharField(db_index=True, max_length=1024),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import datetime
from decimal import Decimal

from django.http import HttpResponseBadRequest, JsonResponse
from django.views.decorators.cache import cache_page

import pybomojo

from bomojo.backends import get_movie_backend
from bomojo.movies.models import Movie, PriceIndex
from bomojo.movies.renderers import render_box_office, render_movie
from bomojo.utils import get_setting, format_money


@cache_page(60 * 60 * 24)
def search(request):
    search_term = request.GET.get('title', '').strip()

    if not search_term:
        return HttpResponseBadRequest('"title" is required')

    if len(search_term) < get_setting('MOVIE_MIN_SEARCH_LENGTH'):
        return JsonResponse({'results': []})

    max_results = get_setting('MOVIE_MAX_SEARCH_RESULTS')
    movies = Movie.objects.search(search_term, max_results)

    if (len(movies) < max_results or
            not any(_is_exact_result(m, search_term) for m in movies)):
        api_results = pybomojo.search_movies(search_term)

        for api_result in api_results:
            movie, created = Movie.objects.get_or_create(
                external_id=api_result['movie_id'], defaults={
                    'title': api_result['title'],
                })
            if created:
                movies.append(movie)
            elif movie.title != api_result['title']:
                # On the off chance the movie's title has changed in the
                # external service, go ahead and update it now.
                movie.title = api_result['title']
                movie.save()

    backend = get_movie_backend()
    results = [_render_movie(m, backend, search_term) for m in movies]

    # Move exact result to the top. This is very inefficient, but it'll get the
    # job done for now. (The more efficient code would be slightly uglier.)
    results = ([r for r in results if r['exact']] +
               [r for r in results if not r['exact']])[:max_results]

    return JsonResponse({'results': results})


@cache_page(60 * 15)
def box_office(request, movie_id):
    try:
        box_office = pybomojo.get_box_office(
            get_movie_backend().parse_movie_id(movie_id))

        if request.GET.get('adjusted', '0') != '0':
            results = box_office.get('box_office', [])
            if len(results) == 0:
                return JsonResponse(render_box_office(box_office, movie_id))

            release_year = min([datetime.strptime(r['date'], '%b. %d, %Y').year
                                for r in results])
            inflation_map = {
                (pi.year, pi.month): pi.value
                for pi in PriceIndex.objects.filter(year__gte=release_year)
            }

            if len(inflation_map) == 0:
                return JsonResponse({
                    'errors': {
                        'adjusted': ['Inflation data is not available.']
                    }
                }, status=500)

            today = datetime.now()
            latest_cpi = inflation_map[sorted(inflation_map.keys())[-1]]

            def adjust_result(result, index):
                date = datetime.strptime(result['date'], '%b. %d, %Y')
                cpi = inflation_map.get((date.year, date.month), latest_cpi)
                result['gross'] = format_money(Decimal(result['gross']) *
                                               latest_cpi / cpi)
                if index > 0:
                    previous_result = results[index - 1]
                    result['cumulative'] = (result['gross'] +
                                            previous_result['cumulative'])
                else:
                    result['cumulative'] = result['gross']

                return result

            for i, result in enumerate(results):
                adjust_result(result, i)

        return JsonResponse(render_box_office(box_office, movie_id))
    except pybomojo.MovieNotFound as e:
        return JsonResponse({
            'errors': {
                'movie': [str(e)]
            }
        }, status=404)


def _render_movie(movie, backend, search_term):
    result = render_movie(movie, backend)
    result['exact'] = _is_exact_result(movie, search_term)
    return result


def _is_exact_result(movie, search_term):
    return movie.title.lower() == search_term



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from bomojo.tests import TestCase


class MatchupTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.bob = cls.create_user('bob')
        cls.alice = cls.create_user('alice')

    def test_str(self):
        matchup = self.create_matchup(self.bob, ['foo', 'bar'])
        self.assertEqual('bob/foo-vs-bar', str(matchup))

    def test_repr(self):
        matchup = self.create_matchup(self.bob, ['foo', 'bar'])
        self.assertEqual('<Matchup: bob/foo-vs-bar>', repr(matchup))

    def test_matchup_populates_slug_on_save(self):
        matchup = self.bob.matchups.create(title='foo vs bar',
                                           movies=['foo', 'bar'])
        self.assertEqual('foo-vs-bar', matchup.slug)

    def test_slug_is_always_unique(self):
        bob_matchup = self.bob.matchups.create(title='foo vs bar',
                                               movies=['foo', 'bar'])
        alice_matchup = self.alice.matchups.create(title='foo vs bar',
                                                   movies=['foo', 'bar'])

        self.assertIn('foo-vs-bar', bob_matchup.slug)
        self.assertIn('foo-vs-bar', alice_matchup.slug)
        self.assertNotEqual(bob_matchup.slug, alice_matchup.slug)

    def test_slug_does_not_change(self):
        matchup = self.bob.matchups.create(title='foo vs bar',
                                           movies=['foo', 'bar'])
        self.assertEqual('foo-vs-bar', matchup.slug)

        matchup.title = 'bar vs foo'
        matchup.save()
        matchup.refresh_from_db()
        self.assertEqual('foo-vs-bar', matchup.slug)



from django.test import override_settings

from bomojo.utils import get_avatar_url


def test_get_avatar_url():
    assert (get_avatar_url('daniel.tao@gmail.com') ==
        'https://gravatar.com/avatar/3c9c19ca551799cf691fddaae5056e55?size=32&d=retro')


def test_get_avatar_url_custom_size():
    assert (get_avatar_url('daniel.tao@gmail.com', size=64) ==
        'https://gravatar.com/avatar/3c9c19ca551799cf691fddaae5056e55?size=64&d=retro')


def test_get_avatar_url_custom_default():
    assert (get_avatar_url('daniel.tao@gmail.com', default='identicon') ==
        'https://gravatar.com/avatar/3c9c19ca551799cf691fddaae5056e55?size=32&d=identicon')


@override_settings(DEFAULT_AVATAR_SIZE=48, DEFAULT_AVATAR_STYLE='robohash')
def test_get_avatar_url_uses_django_settings():
    assert (get_avatar_url('daniel.tao@gmail.com') ==
        'https://gravatar.com/avatar/3c9c19ca551799cf691fddaae5056e55?size=48&d=robohash')


@override_settings(DEFAULT_AVATAR_SIZE=48, DEFAULT_AVATAR_STYLE='robohash')
def test_get_avatar_url_parameters_override_settings():
    assert (get_avatar_url('daniel.tao@gmail.com', size=64, default='identicon') ==
        'https://gravatar.com/avatar/3c9c19ca551799cf691fddaae5056e55?size=64&d=identicon')



"""
bomojo test backends

This module exists to provide example custom backends, to test the library's
support for customizing certain behavior.
"""

from bomojo.backends import AbstractMovieBackend, AbstractUserBackend


class FeatureBobBackend(AbstractUserBackend):
    def is_featured_contributor(self, user):
        return user.username == 'bob'


class UpperToLowerMovieBackend(AbstractMovieBackend):
    def format_external_id(self, external_id):
        return external_id.lower()

    def parse_movie_id(self, movie_id):
        return movie_id.upper()



from django.conf.urls import url

from bomojo.movies import views

urlpatterns = [
    url(r'^search', views.search, name='search'),
    url(r'^(?P<movie_id>[^/]+)/boxoffice', views.box_office, name='box-office'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-27 04:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def associate_matchups_with_users(apps, schema_editor):
    Matchup = apps.get_model('matchups', 'Matchup')
    User = apps.get_model(settings.AUTH_USER_MODEL)

    for matchup in Matchup.objects.filter(email__isnull=False):
        matchup.user = User.objects.get(email=matchup.email)
        matchup.save()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('matchups', '0004_matchup_period_validator'),
    ]

    operations = [
        # Introduce the user field as nullable at first.
        migrations.AddField(
            model_name='matchup',
            name='user',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='matchups', to=settings.AUTH_USER_MODEL),
        ),

        # Associate existing matchups with users by e-mail address.
        migrations.RunPython(associate_matchups_with_users),

        # Now add a non-null constraint.
        migrations.AlterField(
            model_name='matchup',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='matchups', to=settings.AUTH_USER_MODEL),
        ),

        # Finally, remove the now-redundant e-mail field.
        migrations.RemoveField(
            model_name='matchup',
            name='email',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import os

from django.test import override_settings

from freezegun import freeze_time

from bomojo.matchups.models import Matchup
from bomojo.tests import TestCase
from bomojo.utils import get_avatar_url


class CreateTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user = cls.create_user('joe')
        cls.avatar_url = get_avatar_url(cls.user.email)

        cls.create_movie('Foo')
        cls.create_movie('Foo 2', external_id='foo2.htm')
        cls.create_movie('Foo 3', external_id='foo3.htm')
        cls.create_movie('Bar')

    def setUp(self):
        super().setUp()
        self.client.force_login(self.user)

    @freeze_time('2018-03-10 22:17:00')
    def test_happy_path(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': ['foo', 'bar']
        }), content_type='application/json')

        self.assertEqual(201, response.status_code)
        self.assertEqual(response.json(), {
            'creator': {
                'username': 'joe',
                'avatar': self.avatar_url
            },
            'slug': 'foo-vs-bar',
            'title': 'foo vs bar',
            'image_url': None,
            'description': '',
            'movies': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo'
                },
                {
                    'title': 'Bar',
                    'movie_id': 'bar'
                }
            ],
            'period': '',
            'featured': False,
            'created': '2018-03-10T22:17:00+00:00',
            'updated': '2018-03-10T22:17:00+00:00'
        })

        matchup = Matchup.objects.get(slug='foo-vs-bar')
        self.assertEqual('foo vs bar', matchup.title)
        self.assertEqual(['foo.htm', 'bar.htm'], matchup.movies)

        # Ensure fields not supplied are blank.
        self.assertEqual('', matchup.description)
        self.assertEqual('', matchup.period)

    @freeze_time('2018-03-10 22:17:00')
    def test_happy_path_all_fields(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo trilogy',
            'description': 'The entire foo trilogy',
            'movies': ['foo', 'foo2', 'foo3'],
            'period': '90d'
        }), content_type='application/json')

        self.assertEqual(201, response.status_code)
        self.assertEqual(response.json(), {
            'creator': {
                'username': 'joe',
                'avatar': self.avatar_url
            },
            'slug': 'foo-trilogy',
            'title': 'foo trilogy',
            'image_url': None,
            'description': 'The entire foo trilogy',
            'movies': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo'
                },
                {
                    'title': 'Foo 2',
                    'movie_id': 'foo2'
                },
                {
                    'title': 'Foo 3',
                    'movie_id': 'foo3'
                }
            ],
            'period': '90d',
            'featured': False,
            'created': '2018-03-10T22:17:00+00:00',
            'updated': '2018-03-10T22:17:00+00:00'
        })

        matchup = Matchup.objects.get(slug='foo-trilogy')
        self.assertEqual('foo trilogy', matchup.title)
        self.assertEqual('The entire foo trilogy', matchup.description)
        self.assertEqual(['foo.htm', 'foo2.htm', 'foo3.htm'], matchup.movies)
        self.assertEqual('90d', matchup.period)

    def test_requires_title_param(self):
        response = self.client.post('/matchups/', json.dumps({
            'movies': ['foo', 'bar']
        }), content_type='application/json')
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {
            'errors': {
                'title': ['This field is required.']
            }
        })

    def test_title_cannot_be_blank(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': '',
            'movies': ['foo', 'bar']
        }), content_type='application/json')
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {
            'errors': {
                'title': ['This field is required.']
            }
        })

    def test_requires_movies_param(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar'
        }), content_type='application/json')
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {
            'errors': {
                'movies': ['This field is required.']
            }
        })

    def test_movies_cannot_be_empty(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': []
        }), content_type='application/json')
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json(), {
            'errors': {
                'movies': ['This field is required.']
            }
        })

    def test_handles_duplicate_movies(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': ['foo', 'bar', 'bar']
        }), content_type='application/json')
        self.assertEqual(response.status_code, 201)
        response_data = response.json()

        self.assertEqual('foo vs bar', response_data['title'])
        self.assertEqual(['foo', 'bar'],
                         [m['movie_id'] for m in response_data['movies']])

    def test_401_for_unauthenticated_attempt(self):
        self.client.logout()
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': ['foo', 'bar']
        }), content_type='application/json')

        self.assertEqual(401, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'creator': ['You must log in to create a matchup.']
            }
        })

    def test_400_for_bad_attempt_to_create_featured_matchup(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': ['foo', 'bar'],
            'featured': True
        }), content_type='application/json')

        self.assertEqual(400, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'featured': ['You must be a featured contributor to create '
                             'featured matchups.']
            }
        })

    def test_400_for_invalid_movie_ids(self):
        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs quux',
            'movies': ['foo', 'quux']
        }), content_type='application/json')

        self.assertEqual(400, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'movies': ["One or more of those movies isn't recognized."]
            }
        })

    def test_featured_contributor(self):
        self.user.is_staff = True
        self.user.save()

        response = self.client.post('/matchups/', json.dumps({
            'title': 'foo vs bar',
            'movies': ['foo', 'bar'],
            'featured': True
        }), content_type='application/json')

        self.assertEqual(201, response.status_code)
        self.assertEqual(True, response.json()['featured'])

    def test_featured_contributor_custom_backend(self):
        with override_settings(
                USER_BACKEND='bomojo.tests.backends.FeatureBobBackend'):
            # Our default user (joe) should not be able to create a featured
            # matchup.
            response = self.client.post('/matchups/', json.dumps({
                'title': 'foo vs bar',
                'movies': ['foo', 'bar'],
                'featured': True
            }), content_type='application/json')

            self.assertEqual(400, response.status_code)

            # However, a user named "bob" should be, based on our custom
            # FeatureBobBackend class.
            bob = self.create_user('bob')
            self.client.force_login(bob)
            response = self.client.post('/matchups/', json.dumps({
                'title': 'foo vs bar',
                'movies': ['foo', 'bar'],
                'featured': True
            }), content_type='application/json')

            self.assertEqual(201, response.status_code)


class ShowTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.create_movie('Foo')
        cls.create_movie('Bar')

    @freeze_time('2018-03-10 22:17:00')
    def test_happy_path(self):
        user = self.create_user('joe')
        image = self.create_image('foo-vs-bar.png')
        self.create_matchup(user, ['foo', 'bar'], image=image)

        response = self.client.get('/matchups/foo-vs-bar')
        self.assertEqual(response.json(), {
            'creator': {
                'username': 'joe',
                'avatar': get_avatar_url(user.email)
            },
            'slug': 'foo-vs-bar',
            'title': 'foo vs bar',
            'image_url': '/media/matchups/foo-vs-bar.png',
            'description': 'matchup between foo and bar',
            'movies': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo'
                },
                {
                    'title': 'Bar',
                    'movie_id': 'bar'
                }
            ],
            'period': '30d',
            'featured': False,
            'created': '2018-03-10T22:17:00+00:00',
            'updated': '2018-03-10T22:17:00+00:00'
        })

    def test_404_for_missing_matchup(self):
        response = self.client.get('/matchups/does-not-exist')
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.json(), {
            'errors': {
                'slug': ['That matchup does not exist.']
            }
        })


class UpdateTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.create_movie('Foo')
        cls.create_movie('Bar')

    def test_happy_path(self):
        user = self.create_user('joe')

        with freeze_time('2018-04-28 16:14:00'):
            self.create_matchup(user, ['foo', 'bar'])

        self.client.force_login(user)
        with freeze_time('2018-04-28 16:14:01'):
            response = self.client.put('/matchups/foo-vs-bar', json.dumps({
                'title': 'updated title',
                'description': 'updated description'
            }), content_type='application/json')

        self.assertEqual(200, response.status_code)
        self.assertEqual(response.json(), {
            'creator': {
                'username': 'joe',
                'avatar': get_avatar_url(user.email)
            },
            'slug': 'foo-vs-bar',
            'title': 'updated title',
            'image_url': None,
            'description': 'updated description',
            'movies': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo'
                },
                {
                    'title': 'Bar',
                    'movie_id': 'bar'
                }
            ],
            'period': '30d',
            'featured': False,
            'created': '2018-04-28T16:14:00+00:00',
            'updated': '2018-04-28T16:14:01+00:00'
        })

    def test_404_for_missing_matchup(self):
        user = self.create_user('joe')

        self.client.force_login(user)
        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description'
        }), content_type='application/json')

        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.json(), {
            'errors': {
                'slug': ['That matchup does not exist.']
            }
        })

    def test_401_for_unauthenticated_attempt(self):
        user = self.create_user('joe')
        self.create_matchup(user, ['foo', 'bar'])

        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description'
        }), content_type='application/json')

        self.assertEqual(401, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'creator': ['You must log in to edit your matchups.']
            }
        })

    def test_403_for_unauthorized_attempt(self):
        bob = self.create_user('bob')
        self.create_matchup(bob, ['foo', 'bar'])

        alice = self.create_user('alice')

        self.client.force_login(alice)
        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description'
        }), content_type='application/json')

        self.assertEqual(403, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'creator': ["You cannot edit other people's matchups."]
            }
        })

    def test_400_for_bad_attempt_to_update_featured_matchup(self):
        user = self.create_user('joe')
        self.create_matchup(user, ['foo', 'bar'])

        self.client.force_login(user)
        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description',
            'featured': True
        }), content_type='application/json')

        self.assertEqual(400, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'featured': ['You must be a featured contributor to create '
                             'featured matchups.']
            }
        })

    def test_400_for_invalid_movie_ids(self):
        user = self.create_user('joe')
        self.create_matchup(user, ['foo', 'bar'])

        self.client.force_login(user)
        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description',
            'movies': ['foo', 'bar', 'quux']
        }), content_type='application/json')

        self.assertEqual(400, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'movies': ["One or more of those movies isn't recognized."]
            }
        })

    def test_featured_contributor(self):
        user = self.create_user('joe', is_staff=True)
        self.create_matchup(user, ['foo', 'bar'])

        self.client.force_login(user)
        response = self.client.put('/matchups/foo-vs-bar', json.dumps({
            'title': 'updated title',
            'description': 'updated description',
            'featured': True
        }), content_type='application/json')

        self.assertEqual(200, response.status_code)
        self.assertEqual(True, response.json()['featured'])

    def test_featured_contributor_custom_backend(self):
        user = self.create_user('joe', is_staff=True)
        self.create_matchup(user, ['foo', 'bar'])

        self.client.force_login(user)

        with override_settings(
                USER_BACKEND='bomojo.tests.backends.FeatureBobBackend'):
            # Our default user (joe) should not be able to make a matchup
            # featured.
            response = self.client.put('/matchups/foo-vs-bar', json.dumps({
                'title': 'updated title',
                'description': 'updated description',
                'featured': True
            }), content_type='application/json')

            self.assertEqual(400, response.status_code)

            user.username = 'bob'
            user.save()

            # However, after changing their name to "bob" they should be,
            # based on our custom FeatureBobBackend class.
            response = self.client.put('/matchups/foo-vs-bar', json.dumps({
                'title': 'updated title',
                'description': 'updated description',
                'featured': True
            }), content_type='application/json')

            self.assertEqual(200, response.status_code)


class ListTestCase(TestCase):
    def test_featured_matchups(self):
        user = self.create_user('joe')
        avatar_url = get_avatar_url(user.email)

        with freeze_time('2018-03-10 22:17:00'):
            self.create_matchup(user, ['foo', 'bar'])

        with freeze_time('2018-03-10 22:17:01'):
            self.create_matchup(user, ['foo1', 'foo2', 'foo3'],
                           title='foo trilogy',
                           description='all three foo movies')

        # Initially, no matchups should be returned since none are featured.
        response = self.client.get('/matchups/?featured=true')
        self.assertEqual(response.json(), [])

        Matchup.objects.all().update(featured=True)

        # Now that they're featured, both matchups should be returned.
        response = self.client.get('/matchups/?featured=true')
        self.assertEqual(response.json(), [
            {
                'creator': {
                    'username': 'joe',
                    'avatar': avatar_url
                },
                'slug': 'foo-trilogy',
                'title': 'foo trilogy',
                'image_url': None,
                'description': 'all three foo movies',
                'period': '30d',
                'featured': True,
                'created': '2018-03-10T22:17:01+00:00',
                'updated': '2018-03-10T22:17:01+00:00'
            },
            {
                'creator': {
                    'username': 'joe',
                    'avatar': avatar_url
                },
                'slug': 'foo-vs-bar',
                'title': 'foo vs bar',
                'image_url': None,
                'description': 'matchup between foo and bar',
                'period': '30d',
                'featured': True,
                'created': '2018-03-10T22:17:00+00:00',
                'updated': '2018-03-10T22:17:00+00:00'
            }
        ])

    def test_my_matchups(self):
        bob = self.create_user('bob')
        avatar_url = get_avatar_url(bob.email)

        with freeze_time('2018-04-30 06:47:00'):
            self.create_matchup(bob, ['foo', 'bar'])

        with freeze_time('2018-04-30 06:47:01'):
            self.create_matchup(bob, ['foo1', 'foo2', 'foo3'],
                           title='foo trilogy',
                           description='all three foo movies',
                           featured=True)

        alice = self.create_user('alice')
        self.create_matchup(alice, ['bar', 'baz'], featured=True)

        # Requesting the matchups resource should return the current user's
        # matchups. This means an empty list for an anonymous request.
        response = self.client.get('/matchups/')
        self.assertEqual(200, response.status_code)
        self.assertEqual([], response.json())

        self.client.force_login(bob)
        response = self.client.get('/matchups/')
        self.assertEqual(200, response.status_code)

        # Bob's matchups should not include Alice's matchup, even though it's
        # featured.
        self.assertEqual(response.json(), [
            {
                'creator': {
                    'username': 'bob',
                    'avatar': avatar_url
                },
                'slug': 'foo-trilogy',
                'title': 'foo trilogy',
                'image_url': None,
                'description': 'all three foo movies',
                'period': '30d',
                'featured': True,
                'created': '2018-04-30T06:47:01+00:00',
                'updated': '2018-04-30T06:47:01+00:00'
            },
            {
                'creator': {
                    'username': 'bob',
                    'avatar': avatar_url
                },
                'slug': 'foo-vs-bar',
                'title': 'foo vs bar',
                'image_url': None,
                'description': 'matchup between foo and bar',
                'period': '30d',
                'featured': False,
                'created': '2018-04-30T06:47:00+00:00',
                'updated': '2018-04-30T06:47:00+00:00'
            }
        ])

    def test_matchup_sorting(self):
        user = self.create_user('joe')

        with freeze_time('2018-05-04 19:32:00'):
            matchup1 = self.create_matchup(user, ['foo', 'bar'])

        with freeze_time('2018-05-04 19:32:01'):
            matchup2 = self.create_matchup(user, ['bar', 'baz'])

        with freeze_time('2018-05-05 19:32:00'):
            matchup2.description = 'updated matchup #2'
            matchup2.save()

        with freeze_time('2018-05-05 19:32:01'):
            matchup1.description = 'updated matchup #1'
            matchup1.save()

        self.client.force_login(user)

        def expect_results_from_sort(sort, expected_matchups):
            response = self.client.get(f'/matchups/?sort={sort}')
            self.assertEqual(200, response.status_code)
            self.assertEqual([m.slug for m in expected_matchups],
                             [m['slug'] for m in response.json()])

        expect_results_from_sort('created_on', [matchup1, matchup2])
        expect_results_from_sort('-created_on', [matchup2, matchup1])
        expect_results_from_sort('updated_on', [matchup2, matchup1])
        expect_results_from_sort('-updated_on', [matchup1, matchup2])

        response = self.client.get(f'/matchups/?sort=description')
        self.assertEqual(400, response.status_code)
        self.assertEqual(response.json(), {
            'errors': {
                'sort': ['You can only sort by created_on or updated_on.']
            }
        })

    @override_settings(DEFAULT_MATCHUP_SORTING='created_on')
    def test_default_matchup_sorting(self):
        user = self.create_user('joe')

        with freeze_time('2018-05-04 19:32:00'):
            matchup1 = self.create_matchup(user, ['foo', 'bar'])

        with freeze_time('2018-05-04 19:32:01'):
            matchup2 = self.create_matchup(user, ['bar', 'baz'])

        self.client.force_login(user)

        response = self.client.get('/matchups/')
        self.assertEqual(200, response.status_code)
        self.assertEqual([matchup1.slug, matchup2.slug],
                         [m['slug'] for m in response.json()])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.forms.models import model_to_dict
from django.http import JsonResponse
from django.views import View

from bomojo.matchups.forms import MatchupForm
from bomojo.matchups.models import Matchup
from bomojo.matchups.renderers import render_matchup
from bomojo.utils import get_setting


class MatchupsView(View):
    ALLOWED_SORTING = ('created_on', 'updated_on')

    def get(self, request):
        if request.GET.get('featured', 'false') != 'false':
            matchups = Matchup.objects.filter(featured=True)
        elif request.user.is_authenticated:
            matchups = request.user.matchups
        else:
            return JsonResponse([], safe=False)

        sort = request.GET.get('sort', get_setting('DEFAULT_MATCHUP_SORTING'))
        if not (sort in self.ALLOWED_SORTING or
                (sort.startswith('-') and sort[1:] in self.ALLOWED_SORTING)):
            return JsonResponse({
                'errors': {
                    'sort': ['You can only sort by created_on or updated_on.']
                }
            }, status=400)

        recent_matchups = matchups.order_by(sort)[:20]
        return JsonResponse([render_matchup(matchup, include_movies=False)
                             for matchup in recent_matchups], safe=False)

    def post(self, request):
        if not request.user.is_authenticated:
            return JsonResponse({
                'errors': {
                    'creator': ['You must log in to create a matchup.']
                }
            }, status=401)

        form = MatchupForm(request, request.JSON)
        if form.is_valid():
            matchup = form.save()
            return JsonResponse(render_matchup(matchup), status=201)
        else:
            return JsonResponse({'errors': form.errors}, status=400)


class MatchupView(View):
    def get(self, request, slug):
        try:
            matchup = Matchup.objects.get(slug=slug)
            return JsonResponse(render_matchup(matchup))
        except Matchup.DoesNotExist:
            return JsonResponse({
                'errors': {
                    'slug': ['That matchup does not exist.']
                }
            }, status=404)

    def put(self, request, slug):
        if not request.user.is_authenticated:
            return JsonResponse({
                'errors': {
                    'creator': ['You must log in to edit your matchups.']
                }
            }, status=401)

        try:
            matchup = Matchup.objects.get(slug=slug)
        except Matchup.DoesNotExist:
            return JsonResponse({
                'errors': {
                    'slug': ['That matchup does not exist.']
                }
            }, status=404)

        if matchup.user != request.user:
            return JsonResponse({
                'errors': {
                    'creator': ["You cannot edit other people's matchups."]
                }
            }, status=403)

        data = model_to_dict(matchup)
        data.update(request.JSON)

        form = MatchupForm(request, data, instance=matchup)
        if form.is_valid():
            matchup = form.save()
            return JsonResponse(render_matchup(matchup), status=200)
        else:
            return JsonResponse({'errors': form.errors}, status=400)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-20 14:37
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Matchup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(db_index=True, max_length=254, null=True)),
                ('title', models.CharField(db_index=True, max_length=1024)),
                ('slug', models.CharField(db_index=True, max_length=1024, unique=True)),
                ('description', models.TextField(null=True)),
                ('movies', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=128), size=None)),
                ('period', models.CharField(max_length=32, null=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-25 12:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('movies', '0002_movie_title_not_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='PriceIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField(db_index=True)),
                ('month', models.IntegerField()),
                ('value', models.DecimalField(decimal_places=3, max_digits=12)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='priceindex',
            unique_together=set([('year', 'month')]),
        ),
    ]



import os

import dj_database_url

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'test-secret-key'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.postgres',
    'bomojo.movies',
    'bomojo.matchups',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'bomojo.middleware.JSONMiddleware',
]

# Tests extend the default URL config to expose a login endpoint.
ROOT_URLCONF = 'bomojo.urls'

DATABASES = {
    'default': dj_database_url.config(
        default='postgres://postgres@127.0.0.1:5432/bomojo_test')
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'test-cache',
    }
}

MEDIA_ROOT = os.path.join(BASE_DIR, 'tests', 'data')
MEDIA_URL = '/media/'

# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.cache import cache
from django.test import override_settings

from mock import patch
from pybomojo import MovieNotFound

from bomojo.movies.models import Movie
from bomojo.tests import TestCase


class SearchTestCase(TestCase):
    def setUp(self):
        cache.clear()

    def test_requires_title_param(self):
        response = self.client.get('/movies/search')
        self.assertEqual(response.status_code, 400)

    def test_title_cannot_be_blank(self):
        response = self.client.get('/movies/search?title=%20')
        self.assertEqual(response.status_code, 400)

    @patch('pybomojo.search_movies')
    def test_returns_json_response(self, mock):
        mock.return_value = [
            {
                'title': 'Foo',
                'movie_id': 'foo',
                'exact': True
            }
        ]

        response = self.client.get('/movies/search?title=foo')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['content-type'], 'application/json')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                }
            ]
        })

    @patch('pybomojo.search_movies')
    def test_returns_exact_result_first(self, mock):
        mock.return_value = [
            {
                'title': 'Foo',
                'movie_id': 'foo',
                'exact': False
            },
            {
                'title': 'Fools Rush In',
                'movie_id': 'foolsrushin',
                'exact': True
            }
        ]

        response = self.client.get('/movies/search/?title=fools%20rush%20in')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Fools Rush In',
                    'movie_id': 'foolsrushin',
                    'exact': True
                },
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': False
                }
            ]
        })

    @patch('pybomojo.search_movies')
    def test_formats_movie_ids(self, mock):
        mock.return_value = [
            {
                'title': 'Foo',
                'movie_id': 'foo.htm',
                'exact': True
            }
        ]

        response = self.client.get('/movies/search/?title=foo')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                }
            ]
        })

    @override_settings(
        MOVIE_BACKEND='bomojo.tests.backends.UpperToLowerMovieBackend')
    @patch('pybomojo.search_movies')
    def test_configurable_movie_backend(self, mock):
        mock.return_value = [
            {
                'title': 'Foo',
                'movie_id': 'FOO',
                'exact': True
            }
        ]

        response = self.client.get('/movies/search/?title=foo')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                }
            ]
        })

    @patch('pybomojo.search_movies')
    def test_caches_results(self, mock):
        mock.return_value = []

        response = self.client.get('/movies/search/?title=foo%202')
        self.assertJSONEqual(response.content, {'results': []})

        mock.side_effect = AssertionError(
            'should not have called search_movies again (expected to use '
            'cached result)')

        response = self.client.get('/movies/search/?title=foo%202')
        self.assertJSONEqual(response.content, {'results': []})

    @patch('pybomojo.search_movies')
    def test_saves_results_to_database(self, mock):
        mock.return_value = [
            {
                'title': 'Foo',
                'movie_id': 'foo',
                'exact': True
            },
            {
                'title': 'Foo 2: The Awakening',
                'movie_id': 'foo2',
                'exact': False
            }
        ]

        response = self.client.get('/movies/search/?title=foo')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                },
                {
                    'title': 'Foo 2: The Awakening',
                    'movie_id': 'foo2',
                    'exact': False
                }
            ]
        })

        movies = Movie.objects.filter(title__search='Foo').order_by('title')
        self.assertEqual(2, len(movies))
        self.assertEqual('Foo', movies[0].title)
        self.assertEqual('foo', movies[0].external_id)
        self.assertEqual('Foo 2: The Awakening', movies[1].title)
        self.assertEqual('foo2', movies[1].external_id)

    @override_settings(MOVIE_MAX_SEARCH_RESULTS=2)
    @patch('pybomojo.search_movies')
    def test_searches_database_first(self, mock):
        self.create_movie(title='Foo')
        self.create_movie(title='Foo 2: The Awakening', external_id='foo2')

        mock.side_effect = AssertionError(
            'should not have called search_movies since '
            'MOVIE_MAX_SEARCH_RESULTS were already in the database')

        response = self.client.get('/movies/search/?title=foo')
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                },
                {
                    'title': 'Foo 2: The Awakening',
                    'movie_id': 'foo2',
                    'exact': False
                }
            ]
        })

    @override_settings(MOVIE_MAX_SEARCH_RESULTS=2)
    @patch('pybomojo.search_movies')
    def test_searches_api_after_database(self, mock):
        self.create_movie(title='Foo')

        mock.return_value = [
            {
                'title': 'Foo 2: The Awakening',
                'movie_id': 'foo2',
                'exact': False
            },
            {
                'title': 'Foo 3: Return of Foo',
                'movie_id': 'foo3',
                'exact': False
            }
        ]

        response = self.client.get('/movies/search/?title=foo')

        # Only MOVIE_MAX_SEARCH_RESULTS should be returned.
        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                },
                {
                    'title': 'Foo 2: The Awakening',
                    'movie_id': 'foo2',
                    'exact': False
                }
            ]
        })

    @patch('pybomojo.search_movies')
    def test_handles_title_mismatch_between_api_and_database(self, mock):
        movie = self.create_movie(title='Foo', external_id='foo')

        mock.return_value = [
            {
                'title': 'Foo (2018)',
                'movie_id': 'foo',
                'exact': True
            }
        ]

        response = self.client.get('/movies/search/?title=foo')

        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Foo',
                    'movie_id': 'foo',
                    'exact': True
                }
            ]
        })

        # Also, the database record should have been updated.
        movie.refresh_from_db()
        self.assertEqual('Foo (2018)', movie.title)

    @patch('pybomojo.search_movies')
    def test_handles_partial_match_from_database(self, mock):
        movie = self.create_movie(title='Attack of the Foo', external_id='foo')

        mock.return_value = []

        response = self.client.get('/movies/search/?title=att')

        self.assertJSONEqual(response.content, {
            'results': [
                {
                    'title': 'Attack of the Foo',
                    'movie_id': 'foo',
                    'exact': False
                }
            ]
        })


class BoxOfficeTestCase(TestCase):
    def setUp(self):
        cache.clear()

    @patch('pybomojo.get_box_office')
    def test_returns_json_response(self, mock):
        mock.return_value = {
            'title': 'Foo 3: ',
            'href': 'http://moviewebsite.com/foo3',
            'box_office': [{
                'day': 'Fri',
                'date': 'Jul. 14, 2017',
                'rank': 1,
                'gross': 22100000,
                'theaters': 4022,
                'cumulative': 22100000
            }]
        }

        response = self.client.get('/movies/foo3/boxoffice')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['content-type'], 'application/json')
        self.assertJSONEqual(response.content, {
            'title': 'Foo 3: ',
            'movie_id': 'foo3',
            'href': 'http://moviewebsite.com/foo3',
            'box_office': [{
                'day': 'Fri',
                'date': 'Jul. 14, 2017',
                'rank': 1,
                'gross': 22100000,
                'theaters': 4022,
                'cumulative': 22100000
            }]
        })

    @patch('pybomojo.get_box_office')
    def test_parses_movie_ids(self, mock):
        mock.return_value = {}  # doesn't matter
        self.client.get('/movies/foo/boxoffice')
        mock.assert_called_with('foo.htm')

    @override_settings(
        MOVIE_BACKEND='bomojo.tests.backends.UpperToLowerMovieBackend')
    @patch('pybomojo.get_box_office')
    def test_configurable_movie_backend(self, mock):
        mock.return_value = {}  # doesn't matter
        self.client.get('/movies/foo/boxoffice')
        mock.assert_called_with('FOO')

    @patch('pybomojo.get_box_office')
    def test_caches_results(self, mock):
        mock.return_value = {
            'title': 'Bar',
            'href': 'http://moviewebsite.com/bar',
            'box_office': []
        }

        response = self.client.get('/movies/bar/boxoffice')
        self.assertJSONEqual(response.content, {
            'title': 'Bar',
            'movie_id': 'bar',
            'href': 'http://moviewebsite.com/bar',
            'box_office': []
        })

        mock.side_effect = AssertionError(
            'should not have called get_box_office again (expected to use '
            'cached result)')

        response = self.client.get('/movies/bar/boxoffice')
        self.assertJSONEqual(response.content, {
            'title': 'Bar',
            'movie_id': 'bar',
            'href': 'http://moviewebsite.com/bar',
            'box_office': []
        })

    @patch('pybomojo.get_box_office')
    def test_returns_404_if_movie_not_found(self, mock):
        mock.side_effect = MovieNotFound('foo')

        response = self.client.get('/movies/foo/boxoffice')
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.json(), {
            'errors': {
                'movie': ['"foo" not found']
            }
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-05-10 10:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('matchups', '0007_matchup_datetime_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='matchup',
            name='image',
            field=models.ImageField(null=True, upload_to='matchups/'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from bomojo.matchups.models import Matchup
from bomojo.movies.models import Movie


@admin.register(Matchup)
class MatchupAdmin(admin.ModelAdmin):
    list_display = ('featured', 'title', 'user', 'created_on')


@admin.register(Movie)
class MovieAdmin(admin.ModelAdmin):
    list_display = ('title', 'external_id')



from distutils.core import setup
from setuptools import find_packages

requirements = [
    'Django>=1.11.3,<2',

    # Used for partial indexes in Matchup model
    'django-partial-index>=0.4.0,<0.5',

    # Required for ImageField in Matchup model
    'Pillow>=5.1.0,<6',

    # Require Postgres -- can't use sqlite because some models use
    # django.contrib.postgres.fields.ArrayField
    'psycopg2>=2.7.1,<3',

    # Library for querying box office data
    'pybomojo>=0.1.1,<0.2',

    # Library for getting inflation data
    'pycpi>=0.1.1,<0.2',

    # Create slugs from matchup titles
    'python-slugify>=1.2.4,<2',
]

test_requirements = [
    # Used in test settings
    'dj-database-url==0.4.2',

    # Libraries used by the tests themselves
    'freezegun==0.3.10',
    'mock==2.0.0',

    # Libraries used to discover/run the tests
    'pytest==3.2.0',
    'pytest-django==3.1.2',
]

setup(
    name='bomojo',
    packages=find_packages(),
    version='0.8.1',
    description='Django app for getting movie box office data',
    python_requires='>=3.6',
    install_requires=requirements,
    tests_require=test_requirements,
    extras_require={
        'test': test_requirements
    },
    author='Dan Tao',
    author_email='daniel.tao@gmail.com',
    url='https://bitbucket.org/teamdtao/pybomojo',
    keywords=[],
    classifiers=[],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from random import random

from django.conf import settings
from django.contrib.postgres import fields as postgres_fields
from django.core.validators import RegexValidator
from django.db import models

from partial_index import PartialIndex
from slugify import slugify

from bomojo.movies.models import Movie


class Matchup(models.Model):
    class Meta:
        indexes = [
            models.Index(fields=['user', 'created_on']),
            models.Index(fields=['user', 'updated_on']),
            PartialIndex(fields=['created_on'], unique=False,
                         where='featured = true'),
            PartialIndex(fields=['updated_on'], unique=False,
                         where='featured = true')
        ]

    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='matchups')
    title = models.CharField(blank=False, max_length=1024, db_index=True)
    slug = models.CharField(max_length=1024, db_index=True, unique=True)
    image = models.ImageField(null=True, upload_to='matchups/')
    description = models.TextField(blank=True, default='')
    movies = postgres_fields.ArrayField(models.CharField(blank=False,
                                                         max_length=128))
    period = models.CharField(blank=True, default='', max_length=32,
                              validators=[RegexValidator(r'^\d+[dw]$')])
    featured = models.BooleanField(default=False)
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f'{self.user.username}/{self.slug}'

    def save(self, *args, **kwargs):
        if not self.pk:
            slug_base = slugify(self.title)
            self.slug = slug_base
            while Matchup.objects.filter(slug=self.slug).exists():
                self.slug = '%s-%s' % (slug_base, str(random())[2:])
        super(Matchup, self).save(*args, **kwargs)

    def get_movies(self):
        return Movie.objects.filter(external_id__in=self.movies)



from bomojo.movies.renderers import render_movie
from bomojo.utils import get_avatar_url


def render_matchup(matchup, include_movies=True):
    result = {
        'creator': render_user(matchup.user),
        'slug': matchup.slug,
        'title': matchup.title,
        'image_url': matchup.image.url if matchup.image else None,
        'description': matchup.description,
        'period': matchup.period,
        'featured': matchup.featured,
        'created': matchup.created_on.isoformat(),
        'updated': matchup.updated_on.isoformat()
    }

    if include_movies:
        result['movies'] = [render_movie(movie) for movie
                            in matchup.get_movies()]

    return result


def render_user(user):
    return {
        'username': user.username,
        'avatar': get_avatar_url(user.email)
    }



from django.conf.urls import url

from bomojo.matchups.views import MatchupView, MatchupsView

urlpatterns = [
    url(r'^$', MatchupsView.as_view(), name='matchups'),
    url(r'^(?P<slug>[^/]+)$', MatchupView.as_view(), name='matchup'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from functools import reduce

from django.db import models
from django.db.models import Q

from bomojo.utils import split_on_whitespace


class MovieManager(models.Manager):
    def search(self, search_term, max_results):
        queryset = self.get_queryset()
        matches = queryset.filter(title__search=search_term)

        results = list(matches[:max_results])
        if len(results) == max_results:
            return results

        matches = queryset.filter(
            title__istartswith=search_term).exclude(
            id__in=[r.id for r in results])
        results += list(matches[:(max_results - len(results))])
        if len(results) == max_results:
            return results

        query = reduce(lambda q, term: q & Q(title__icontains=term),
                       split_on_whitespace(search_term), Q())
        matches = queryset.filter(query).exclude(
            id__in=[r.id for r in results])
        results += list(matches[:(max_results - len(results))])
        return results


class Movie(models.Model):
    title = models.CharField(max_length=1024, db_index=True)
    external_id = models.CharField(max_length=2048, db_index=True, unique=True)

    objects = MovieManager()

    def __str__(self):
        return f'{self.title} ({self.external_id})'


class PriceIndex(models.Model):
    class Meta:
        unique_together = [
            ('year', 'month')
        ]

    year = models.IntegerField(db_index=True)
    month = models.IntegerField()
    value = models.DecimalField(max_digits=12, decimal_places=3)

    def __str__(self):
        return f'{self.year}-{self.month:02}: {self.value}'
